VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsFolderBrowse"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Copyright (C) 2005 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Different start locations
Public Enum START_LOCATION
    SL_FLAG_CREATE = &H8000
    SL_FLAG_DONT_VERIFY = &H4000
    SL_ADMINTOOLS = &H30
    SL_ALTSTARTUP = &H1D
    SL_APPDATA = &H1A
    SL_BITBUCKET = &HA
    SL_COMMON_ADMINTOOLS = &H2F
    SL_COMMON_ALTSTARTUP = &H1D
    SL_COMMON_APPDATA = &H23
    SL_COMMON_DESKTOPDIRECTORY = &H19
    SL_COMMON_DOCUMENTS = &H2E
    SL_COMMON_FAVORITES = &H1F
    SL_COMMON_PROGRAMS = &H17
    SL_COMMON_STARTMENU = &H16
    SL_COMMON_STARTUP = &H18
    SL_COMMON_TEMPLATES = &H2D
    SL_CONTROLS = &H3
    SL_COOKIES = &H21
    SL_DESKTOP = &H0
    SL_DESKTOPDIRECTORY = &H10
    SL_DRIVES = &H11
    SL_FAVORITES = &H6
    SL_FONTS = &H14
    SL_HISTORY = &H22
    SL_INTERNET = &H1
    SL_INTERNET_CACHE = &H20
    SL_LOCAL_APPDATA = &H1C
    SL_MYPICTURES = &H27
    SL_NETHOOD = &H13
    SL_NETWORK = &H12
    SL_PERSONAL = &H5
    SL_PRINTERS = &H4
    SL_PRINTHOOD = &H1B
    SL_PROFILE = &H28
    SL_PROGRAM_FILES = &H26
    SL_PROGRAM_FILES_COMMON = &H2B
    SL_PROGRAM_FILES_COMMONX86 = &H2C
    SL_PROGRAM_FILESX86 = &H2A
    SL_PROGRAMS = &H2
    SL_RECENT = &H8
    SL_SENDTO = &H9
    SL_STARTMENU = &HB
    SL_STARTUP = &H7
    SL_SYSTEM = &H25
    SL_SYSTEMX86 = &H29
    SL_TEMPLATES = &H15
    SL_WINDOWS = &H24
End Enum
    
' Different flag-values
Public Enum BIF_FLAGS
    BIF_RETURNONLYFSDIRS = 1
    BIF_DONTGOBELOWDOMAIN = 2
    BIF_RETURNFSANCESTORS = 8
    BIF_EDITBOX = 16
    BIF_VALIDATE = 32
    BIF_NEWDIALOGSTYLE = 64
    BIF_BROWSEINCLUDEURLS = 128
    BIF_NONEWFOLDERBUTTON = 512
    BIF_BROWSEFORCOMPUTER = 4096
    BIF_BROWSEFORPRINTER = 8192
    BIF_BROWSEINCLUDEFILES = 16384
    BIF_SHAREABLE = 32768
End Enum

Enum BrowseResult
    Browse_OK
    Browse_CANCEL
End Enum

' The default maximum lenght of a path
Const MAX_PATH = 260

' Different variables used to customize the browse-form
Public Description As String
Public Filters As String
Public Flags As BIF_FLAGS

' Where to start
Public RootLocation As START_LOCATION
Public RootPath As String

' What to select
Public StartLocation As START_LOCATION
Public StartPath As String

' Variables holding the result
Public ResultPath As String

' The main function which invokes the form
Public Function Browse(Optional hwndOwner As Long) As BrowseResult

    Dim lpIDList As Long, udtBI As BROWSEINFO, Item(1) As ITEMIDLIST, lpItem(1) As Long
    
    ' Firstly assume that we didn't get a OK
    Browse = Browse_CANCEL

    ' Retrieve folders
    lpItem(0) = RetrieveFolder(RootLocation, RootPath, Item(0))
    lpItem(1) = RetrieveFolder(StartLocation, StartPath, Item(1))

    ' Initialize user-defined type
    With udtBI
        .hOwner = hwndOwner
        .pidlRoot = lpItem(0)
        .lpszTitle = Description
        .ulFlags = Flags
        .lParam = lpItem(1)
        .lpfn = GetAddress(AddressOf BrowseCallbackProc)
    End With

    'Show the 'Browse for folder' dialog
    lpIDList = SHBrowseForFolder(udtBI)
    
    ' Then try to get the folder
    If lpIDList Then
        
        ' Retrieve the string
        ResultPath = GetStringFromIDList(lpIDList)
        
        ' Make sure there's something to return
        If LenB(ResultPath) <> 0 Then

            ' Everything went ok
            Browse = Browse_OK
        
        End If
        
    End If
    
    ' Clean up
    CoTaskMemFree lpItem(0)
    CoTaskMemFree lpItem(1)
    
End Function

Private Function RetrieveFolder(Flag As Long, Path As String, Item As ITEMIDLIST) As Long

    ' Fill the IDLIST with the right information
    If LenB(Path) <> 0 Then
    
        ' Create the IDLIST with the correct information
        RetrieveFolder = SHSimpleIDListFromPath(ConvertString(Path))
        
    Else
        ' Retrieve the special folder location
        SHGetSpecialFolderLocation 0, Flag, Item
        
        ' Save the pointer to it
        RetrieveFolder = Item.mkid.cb
        
    End If

End Function

Private Function ConvertString(sText As String) As String

    ' Return the string as default
    ConvertString = sText
    
    ' Now, if we're running in NT, we need to double-buffer a unicode-string, since VB automatically
    ' converts the string to ANSI when a API is called.
    If IsWinNT Then
        ConvertString = StrConv(sText, vbUnicode)
    End If

End Function

Private Function GetStringFromIDList(ByVal lHandle As Long) As String

    Dim iNull As Integer, lpIDList As Long, sPath As String
    
    ' Allocate buffer
    sPath = String$(MAX_PATH, 0)
    
    'Get the path from the IDList
    SHGetPathFromIDList lHandle, sPath
    
    'free the block of memory
    CoTaskMemFree lHandle
    iNull = InStr(sPath, vbNullChar)

    ' See if there's anything left
    If iNull Then

        ' Set the return path
        GetStringFromIDList = Left$(sPath, iNull - 1)

    End If

End Function

Private Sub Class_Initialize()

    ' Set the default value
    Flags = BIF_RETURNONLYFSDIRS

End Sub

